
% Filename: Denoiose_YUV.m
% Operation: denoisinig color images in the luminance-chrominance space 
% using greyscale version of the spatially adaptive ProbShring algorithm from
%
%A. Pizurica and W. Philips, 
%"Estimating probability of presence of a signal of interest in multiresolution single- 
%and multiband image denoising, " IEEE Transactions on Image Processing (in press) 
%
% Author: Aleksandra Pizurica/TELIN/Ghent University
% E-mail: Aleksandra.Pizurica@telin.UGent.be
% Last modified: February 2004.
% Copyright: All software, documentation, and related files in this distribution
%            are Copyright (c) 2003 Aleksandra Pizurica. All rights reserved.
%
%
% This denoising program uses wavelet decomposition and reconstruction functions
% developed by Rice University, Houston, Texas and its contributors.
%
% This software is distributed and licensed to you on a non-exclusive 
% basis, free-of-charge. Redistribution and use in source and binary forms, 
% with or without modification, are permitted provided that the above 
% copyright notice is clearly maintained.


clear all;
close all;
 
Ref=input('Reference noise-free image available? (y/n) ','s');

% Read reference, noise-free image
if (Ref=='y')
    InputType=0;
    while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
     InputType=input('\nReference image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bmp"\n>>  ');
    end
    imload;
    Xclean=SCAN;
    clear SCAN;
    [M,N]=size(Xclean(:,:,1));
  
    Choice=input('Choose 1 or 2:\n1. An arbitrary noise variance\n2. A stored noisy image\n ');

    if(Choice==1)
       NoiseStd=input('Standard Deviation of added noise: ');
       Xnoisy=Xclean+randn(M,N,3)*NoiseStd;
       clean_image=input('Reference image name: ','s');
       image_name=strcat(clean_image,'_std',num2str(NoiseStd));
    else
       InputType=0;
       while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
         InputType=input('\nNoisy image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bmp"\n>>  ');
       end
       imload;
       Xnoisy=SCAN;
       clear SCAN;
  
       %Extract the image name from the filename
       image_name=filename(1:(length(filename)-4))
       
    end
  
else % No reference noise-free image image available

    % Read noisy image
    InputType=0;
    while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
        InputType=input('\nNoisy image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bin"\n>>  ');
    end
    imload;
    Xnoisy=SCAN;
    clear SCAN;
    imagesc(uint8(Xnoisy));
    colormap(gray);

    %Extract the image name from the filename
    image_name=filename(1:(length(filename)-4))
    
end
 
[M,N]=size(Xnoisy(:,:,1)); 

figure
imagesc(uint8(Xnoisy));
title('Input image');


%======================================================
% Parameters

% Choose a wavelet
 wtypeN=0;
  while((wtypeN~=1) & (wtypeN~=2) & (wtypeN~=3) & (wtypeN~=4) & (wtypeN~=5))
    wtypeN=input('select a wavelet: 1. (db2), 2. (db4),  3. (sym2), 4. (sym4) or 5. (sym8) ');
 end
 
 if(wtypeN==1)
    wtype='db2'
    [LO_D,HI_D,LO_R,HI_R] = wfilters('db2');
    h=LO_R;
 elseif(wtypeN==2)
    wtype='db4'
    [LO_D,HI_D,LO_R,HI_R] = wfilters('db4');
    h=LO_R;
 elseif(wtypeN==3)
    wtype='sym2'
    [LO_D,HI_D,LO_R,HI_R] = wfilters('sym2');
    h=LO_R;
 elseif(wtypeN==4)
    wtype='sym4'
    [LO_D,HI_D,LO_R,HI_R] = wfilters('sym4');
    h=LO_R;
 elseif(wtypeN==5)
    wtype='sym8'
    [LO_D,HI_D,LO_R,HI_R] = wfilters('sym8');
    h=LO_R;
 end

W=2;
SigIntDEF=1;

%=====================================================  

% Transformation from RGB to YUV space

Y =  0.299*Xnoisy(:,:,1) + 0.587*Xnoisy(:,:,2) + 0.114*Xnoisy(:,:,3);
U = -0.147*Xnoisy(:,:,1) - 0.289*Xnoisy(:,:,2) + 0.436*Xnoisy(:,:,3);
V =  0.615*Xnoisy(:,:,1) - 0.515*Xnoisy(:,:,2) - 0.100*Xnoisy(:,:,3);


[clp,chor,cver,cdiag]=dwt2(Y,'db1');
NoiseStd=Thr_univ(cdiag);
Yden=genlikSI_fun(Y,NoiseStd,W,h,SigIntDEF);
figure
subplot(2,1,1),imagesc(Y);
subplot(2,1,2),imagesc(Yden);
colormap(gray);

[clp,chor,cver,cdiag]=dwt2(U,'db1');
NoiseStd=Thr_univ(cdiag);
Uden=genlikSI_fun(U,NoiseStd,W,h,SigIntDEF);
figure
subplot(2,1,1),imagesc(U);
subplot(2,1,2),imagesc(Uden);
colormap(gray);

[clp,chor,cver,cdiag]=dwt2(V,'db1');
NoiseStd=Thr_univ(cdiag);
Vden=genlikSI_fun(V,NoiseStd,W,h,SigIntDEF);
figure
subplot(2,1,1),imagesc(V);
subplot(2,1,2),imagesc(Vden);
colormap(gray);

clear Y U V;


% Transform YUV to RGB
Xden(:,:,1)= Yden + 0.000*Uden + 1.140*Vden;
Xden(:,:,2)= Yden - 0.396*Uden - 0.581*Vden;
Xden(:,:,3)= Yden + 2.029*Uden + 0.000*Vden;

figure
imagesc(uint8(Xden));
colormap(gray);
title('Reconstructed image');
   
   
if(Ref=='y')
  
  MSE=sum(sum(sum((Xclean-Xden).^2)))/(M*N*3);
  MSEin=sum(sum(sum((Xclean-Xnoisy).^2)))/(M*N*3);


PSNR=10*log10(255^2/MSE)
PSNRin=10*log10(255^2/MSEin)
end

   

MSE_R=sum(sum((Xclean(:,:,1)-Xden(:,:,1)).^2))/(M*N);
PSNR_R=10*log10(255^2/MSE_R)


MSE_G=sum(sum((Xclean(:,:,2)-Xden(:,:,2)).^2))/(M*N);
PSNR_G=10*log10(255^2/MSE_G)

MSE_R=sum(sum((Xclean(:,:,1)-Xden(:,:,1)).^2))/(M*N);
PSNR_R=10*log10(255^2/MSE_R);


MSE_B=sum(sum((Xclean(:,:,3)-Xden(:,:,3)).^2))/(M*N);
PSNR_B=10*log10(255^2/MSE_B)



    
